@@ -9,7 +9,7 @@ from isort.settings import DEFAULT_CONFIG, Config
 from . import output, parse
 from .exceptions import FileSkipComment
 from .format import format_natural, remove_whitespace
-from .settings import FILE_SKIP_COMMENTS
+from .settings import FILE_SKIP_RE
 
 CIMPORT_IDENTIFIERS = ("cimport ", "cimport*", "from.cimport")
 IMPORT_START_IDENTIFIERS = ("from ", "from.import", "import ", "import*") + CIMPORT_IDENTIFIERS
@@ -150,12 +150,11 @@ def process(
             if stripped_line and not line_separator:
                 line_separator = line[len(line.rstrip()) :].replace(" ", "").replace("\t", "")
 
-            for file_skip_comment in FILE_SKIP_COMMENTS:
-                if file_skip_comment in line:
-                    if raise_on_skip:
-                        raise FileSkipComment("Passed in content")
-                    isort_off = True
-                    skip_file = True
+            if FILE_SKIP_RE.match(line):
+                if raise_on_skip:
+                    raise FileSkipComment("Passed in content")
+                isort_off = True
+                skip_file = True
 
             if not in_quote:
                 if stripped_line == "# isort: off":
@@ -176,10 +175,13 @@ def process(
                 (index == 0 or (index in (1, 2) and not contains_imports))
                 and stripped_line.startswith("#")
                 and stripped_line not in config.section_comments
+                and stripped_line not in CODE_SORT_COMMENTS
             ):
                 in_top_comment = True
             elif in_top_comment and (
-                not line.startswith("#") or stripped_line in config.section_comments
+                not line.startswith("#")
+                or stripped_line in config.section_comments
+                or stripped_line in CODE_SORT_COMMENTS
             ):
                 in_top_comment = False
                 first_comment_index_end = index - 1
@@ -246,7 +248,10 @@ def process(
                     else:
                         code_sorting_section += line
                         line = ""
-                elif stripped_line in config.section_comments:
+                elif (
+                    stripped_line in config.section_comments
+                    or stripped_line in config.section_comments_end
+                ):
                     if import_section and not contains_imports:
                         output_stream.write(import_section)
                         import_section = line
@@ -350,7 +355,7 @@ def process(
                 next_import_section = ""
 
             if import_section:
-                if add_imports and not indent:
+                if add_imports and (contains_imports or not config.append_only) and not indent:
                     import_section = (
                         line_separator.join(add_imports) + line_separator + import_section
                     )
@@ -457,6 +462,7 @@ def _indented_config(config: Config, indent: str) -> Config:
         wrap_length=max(config.wrap_length - len(indent), 0),
         lines_after_imports=1,
         import_headings=config.import_headings if config.indented_import_headings else {},
+        import_footers=config.import_footers if config.indented_import_headings else {},
     )
 
 
