@@ -41,22 +41,20 @@ from .exceptions import (
 from .profiles import profiles
 from .sections import DEFAULT as SECTION_DEFAULTS
 from .sections import FIRSTPARTY, FUTURE, LOCALFOLDER, STDLIB, THIRDPARTY
+from .utils import Trie
 from .wrap_modes import WrapModes
 from .wrap_modes import from_string as wrap_mode_from_string
 
 if TYPE_CHECKING:
-    toml: Any
+    tomli: Any
 else:
-    from ._vendored import toml
+    from ._vendored import tomli
 
 _SHEBANG_RE = re.compile(br"^#!.*\bpython[23w]?\b")
 CYTHON_EXTENSIONS = frozenset({"pyx", "pxd"})
 SUPPORTED_EXTENSIONS = frozenset({"py", "pyi", *CYTHON_EXTENSIONS})
 BLOCKED_EXTENSIONS = frozenset({"pex"})
-FILE_SKIP_COMMENTS: Tuple[str, ...] = (
-    "isort:" + "skip_file",
-    "isort: " + "skip_file",
-)  # Concatenated to avoid this file being skipped
+FILE_SKIP_RE = re.compile(r"^#?\s?isort:\s?skip_file")
 MAX_CONFIG_SEARCH_DEPTH: int = 25  # The number of parent directories to for a config file within
 STOP_CONFIG_SEARCH_ON_DIRS: Tuple[str, ...] = (".git", ".hg")
 VALID_PY_TARGETS: Tuple[str, ...] = tuple(
@@ -102,6 +100,7 @@ CONFIG_SECTIONS: Dict[str, Tuple[str, ...]] = {
 FALLBACK_CONFIG_SECTIONS: Tuple[str, ...] = ("isort", "tool:isort", "tool.isort")
 
 IMPORT_HEADING_PREFIX = "import_heading_"
+IMPORT_FOOTER_PREFIX = "import_footer_"
 KNOWN_PREFIX = "known_"
 KNOWN_SECTION_MAPPING: Dict[str, str] = {
     STDLIB: "STANDARD_LIBRARY",
@@ -173,10 +172,12 @@ class _Config:
     single_line_exclusions: Tuple[str, ...] = ()
     default_section: str = THIRDPARTY
     import_headings: Dict[str, str] = field(default_factory=dict)
+    import_footers: Dict[str, str] = field(default_factory=dict)
     balanced_wrapping: bool = False
     use_parentheses: bool = False
     order_by_type: bool = True
     atomic: bool = False
+    lines_before_imports: int = -1
     lines_after_imports: int = -1
     lines_between_sections: int = 1
     lines_between_types: int = 0
@@ -245,7 +246,7 @@ class _Config:
             if sys.version_info.major == 2 and sys.version_info.minor <= 6:
                 py_version = "2"
             elif sys.version_info.major == 3 and (
-                sys.version_info.minor <= 5 or sys.version_info.minor >= 9
+                sys.version_info.minor <= 5 or sys.version_info.minor >= 10
             ):
                 py_version = "3"
             else:
@@ -297,6 +298,7 @@ class Config(_Config):
     ):
         self._known_patterns: Optional[List[Tuple[Pattern[str], str]]] = None
         self._section_comments: Optional[Tuple[str, ...]] = None
+        self._section_comments_end: Optional[Tuple[str, ...]] = None
         self._skips: Optional[FrozenSet[str]] = None
         self._skip_globs: Optional[FrozenSet[str]] = None
         self._sorting_function: Optional[Callable[..., List[str]]] = None
@@ -307,6 +309,7 @@ class Config(_Config):
             config_vars["py_version"] = config_vars["py_version"].replace("py", "")
             config_vars.pop("_known_patterns")
             config_vars.pop("_section_comments")
+            config_vars.pop("_section_comments_end")
             config_vars.pop("_skips")
             config_vars.pop("_skip_globs")
             config_vars.pop("_sorting_function")
@@ -381,6 +384,7 @@ class Config(_Config):
 
         known_other = {}
         import_headings = {}
+        import_footers = {}
         for key, value in tuple(combined_config.items()):
             # Collect all known sections beyond those that have direct entries
             if key.startswith(KNOWN_PREFIX) and key not in (
@@ -417,6 +421,8 @@ class Config(_Config):
                         )
             if key.startswith(IMPORT_HEADING_PREFIX):
                 import_headings[key[len(IMPORT_HEADING_PREFIX) :].lower()] = str(value)
+            if key.startswith(IMPORT_FOOTER_PREFIX):
+                import_footers[key[len(IMPORT_FOOTER_PREFIX) :].lower()] = str(value)
 
             # Coerce all provided config values into their correct type
             default_value = _DEFAULT_SETTINGS.get(key, None)
@@ -495,6 +501,10 @@ class Config(_Config):
             for import_heading_key in import_headings:
                 combined_config.pop(f"{IMPORT_HEADING_PREFIX}{import_heading_key}")
             combined_config["import_headings"] = import_headings
+        if import_footers:
+            for import_footer_key in import_footers:
+                combined_config.pop(f"{IMPORT_FOOTER_PREFIX}{import_footer_key}")
+            combined_config["import_footers"] = import_footers
 
         unsupported_config_errors = {}
         for option in set(combined_config.keys()).difference(
@@ -538,7 +548,7 @@ class Config(_Config):
             return bool(_SHEBANG_RE.match(line))
 
     def _check_folder_gitignore(self, folder: str) -> Optional[Path]:
-        env = {"LANG": "C.UTF-8"}
+        env = {**os.environ, "LANG": "C.UTF-8"}
         try:
             topfolder_result = subprocess.check_output(  # nosec # skipcq: PYL-W1510
                 ["git", "-C", folder, "rev-parse", "--show-toplevel"], encoding="utf-8", env=env
@@ -653,6 +663,14 @@ class Config(_Config):
         self._section_comments = tuple(f"# {heading}" for heading in self.import_headings.values())
         return self._section_comments
 
+    @property
+    def section_comments_end(self) -> Tuple[str, ...]:
+        if self._section_comments_end is not None:
+            return self._section_comments_end
+
+        self._section_comments_end = tuple(f"# {footer}" for footer in self.import_footers.values())
+        return self._section_comments_end
+
     @property
     def skips(self) -> FrozenSet[str]:
         if self._skips is not None:
@@ -765,19 +783,49 @@ def _find_config(path: str) -> Tuple[str, Dict[str, Any]]:
     return (path, {})
 
 
+@lru_cache()
+def find_all_configs(path: str) -> Trie:
+    """
+    Looks for config files in the path provided and in all of its sub-directories.
+    Parses and stores any config file encountered in a trie and returns the root of
+    the trie
+    """
+    trie_root = Trie("default", {})
+
+    for (dirpath, _, _) in os.walk(path):
+        for config_file_name in CONFIG_SOURCES:
+            potential_config_file = os.path.join(dirpath, config_file_name)
+            if os.path.isfile(potential_config_file):
+                config_data: Dict[str, Any]
+                try:
+                    config_data = _get_config_data(
+                        potential_config_file, CONFIG_SECTIONS[config_file_name]
+                    )
+                except Exception:
+                    warn(f"Failed to pull configuration information from {potential_config_file}")
+                    config_data = {}
+
+                if config_data:
+                    trie_root.insert(potential_config_file, config_data)
+                    break
+
+    return trie_root
+
+
 @lru_cache()
 def _get_config_data(file_path: str, sections: Tuple[str]) -> Dict[str, Any]:
     settings: Dict[str, Any] = {}
 
-    with open(file_path, encoding="utf-8") as config_file:
-        if file_path.endswith(".toml"):
-            config = toml.load(config_file)
-            for section in sections:
-                config_section = config
-                for key in section.split("."):
-                    config_section = config_section.get(key, {})
-                settings.update(config_section)
-        else:
+    if file_path.endswith(".toml"):
+        with open(file_path, "rb") as bin_config_file:
+            config = tomli.load(bin_config_file)
+        for section in sections:
+            config_section = config
+            for key in section.split("."):
+                config_section = config_section.get(key, {})
+            settings.update(config_section)
+    else:
+        with open(file_path, encoding="utf-8") as config_file:
             if file_path.endswith(".editorconfig"):
                 line = "\n"
                 last_position = config_file.tell()
@@ -790,22 +838,22 @@ def _get_config_data(file_path: str, sections: Tuple[str]) -> Dict[str, Any]:
 
             config = configparser.ConfigParser(strict=False)
             config.read_file(config_file)
-            for section in sections:
-                if section.startswith("*.{") and section.endswith("}"):
-                    extension = section[len("*.{") : -1]
-                    for config_key in config.keys():
-                        if (
-                            config_key.startswith("*.{")
-                            and config_key.endswith("}")
-                            and extension
-                            in map(
-                                lambda text: text.strip(), config_key[len("*.{") : -1].split(",")  # type: ignore # noqa
-                            )
-                        ):
-                            settings.update(config.items(config_key))
+        for section in sections:
+            if section.startswith("*.{") and section.endswith("}"):
+                extension = section[len("*.{") : -1]
+                for config_key in config.keys():
+                    if (
+                        config_key.startswith("*.{")
+                        and config_key.endswith("}")
+                        and extension
+                        in map(
+                            lambda text: text.strip(), config_key[len("*.{") : -1].split(",")  # type: ignore # noqa
+                        )
+                    ):
+                        settings.update(config.items(config_key))
 
-                elif config.has_section(section):
-                    settings.update(config.items(section))
+            elif config.has_section(section):
+                settings.update(config.items(section))
 
     if settings:
         settings["source"] = file_path
